/*
****************************************************************************
This version of the firmware has the most popular 45 POWER codes for Europe.
****************************************************************************
*/

/*
TV-B-Gone
Firmware
for use with ATmega8L
Malte Block
29-Jun-09

Distributed under Creative Commons 2.5 -- Attib & Share Alike
*/

/*
Ladyada has a great website about the MiniPOV kit, from which this project was hacked:
http://ladyada.net/make/minipov3/index.html
Her website also has a user forum, where people can ask and answer questions by people building
various projects, including this one:
http://www.ladyada.net/forums/
(Click on the link for MiniPOV).
*/

/*
This project transmits a bunch of TV POWER codes, one right after the other, 
with a pause in between each.  (To have a visible indication that it is 
transmitting, it also pulses a Red  LED once each time a POWER code is 
transmitted.)  That is all TV-B-Gone does.  The tricky part of TV-B-Gone 
was collecting all of the POWER codes, and getting rid of the duplicates and 
near-duplicates (because if there is a duplicate, then one POEWR code will 
turn a TV off, and the duplicate will turn it on again (which we certainly 
do not want).  I have compiled the top-40 most popular codes with the 
duplicates eliminated, both for North America (which is the same as Asia, as 
far as POWER codes are concerned -- even though much of Asia USES PAL video) 
and for Europe (which works for Australia, New Zealand, the Middle East, and 
other parts of the world that use PAL video).
*/

/*
FUSES: HIGH = 0xD9, LOW = 0xBF
*/

.include "m8def.inc"                     ; ATmega8L

.equ XTAL = 8000000                      ; 8MHz (Quarz)!!!!
.equ LED_PORT = PORTB                    ; alles an Port B
.equ LED_PORT_DDR =DDRB                  ; Datenrichtung Port B
.equ IR_LED_PIN = 1                      ; OC1A an Pin 1 IR_Led
.equ V_LED_PIN =0                        ; V_Led an Pin 0
 
.def Timer1 = r16                        ; fr 10s Softtimer
.def Timer2 = r17                        ; 16-Bir Softtimer LSB + Anzeit
.def Timer3 = r18                        ; 16-Bir Softtimer MSB + Anzeit
.def Timer4 = r19                        ; Auszeit LSB 
.def Timer5 = r20                        ; Auszeit MSB

.def temp1  = r21

; *********** Beschaltung ATmega 8 L ***************
; * PB0                  --> normale LED (L-Aktiv) *                    
; * PB1 (OC1A)           --> IR-LED      (H-Aktiv) *
; * PB6 (XTAL1/TOSC1)    --> Quarz + 15nF -> GND   *
; * PB7 (XTAL2/TOSC2)    --> Quarz + 15nF -> GND   *
; * PC6 (RESET)          --> Taster -> GND         *  
; **************************************************
.org 0x0000
Reset:
        ldi    temp1, LOW(RAMEND)        ; LOW-Byte der obersten RAM-Adresse
        out    SPL, temp1
        ldi    temp1, HIGH(RAMEND)       ; HIGH-Byte der obersten RAM-Adresse
        out    SPH, temp1
           
        ldi    temp1,  (1<<SE) | (1<<SM1); Sleep Mode initialisieren
        out    MCUCR, temp1              ; Power-Down       
	    
        ; IR_Led aus (H-Aktiv), V_Led aus (L-Aktiv)
        ldi    temp1, (0<<IR_LED_PIN) | (1<<V_LED_PIN)
        out    LED_PORT, temp1
        ; Ausgnge Definieren 
		ldi    temp1, (1<<IR_LED_PIN) | (1<<V_LED_PIN)
        out    LED_PORT_DDR, temp1 
        ; los gehts
        rcall  Flash_IR_Code
        ; alle Code gesendet, LED Port auf Eingang 
        clr    temp1                     ; Port B auf Eingang
        out    LED_PORT_DDR, temp1

        sleep
        rjmp   Reset


; ************ Soft Timer ******************************************
Wait_10_Mikrosekunden:                   ; 4  Takte Aufruf 
;       80 Zyklen = 10s bei 8Mz:
        ldi    Timer1, 25                ; 1  Takt Zeit laden
Wait_10_Mikrosekunden_loop:  
        dec    Timer1                    ; 1  Takt
        brne   Wait_10_Mikrosekunden_loop; 2  Takte (1 Takt)
                                         ; -------- 
                                         ; 72 Takte

        ret                              ; 4  Takte Rcksprung
                                         ; --------
                                         ; 80 Takte 

Wait_10us_Timer2_Timer3:   ; Timer2 = LSB , Timer3 = MSB
        rcall   Wait_10_Mikrosekunden
        dec     Timer2
        brne    Wait_10us_Timer2_Timer3
        subi    Timer3, 1                ; dec Timer3
		brcc    Wait_10us_Timer2_Timer3  ; cpi Timer3, 0xff
        ret                              ; brne .....
                                         ; ret

; ************ Flash visible Led **********************************
Flash_visible_Led:
        ; V_Led an
        cbi    LED_PORT, V_LED_PIN
        ; warte 20 ms
        ldi    Timer2, LOW(2000)
        ldi    Timer3, HIGH(2000)
        rcall  Wait_10us_Timer2_Timer3
        ; V_Led aus
        sbi    LED_PORT, V_LED_PIN
        ret
		
Flash_visible_Led_4x:
        rcall  Flash_visible_Led
        ; warte 50 ms
        ldi    Timer2, LOW (5000)
        ldi    Timer3, HIGH(5000)
        rcall  Wait_10us_Timer2_Timer3
        rcall  Flash_visible_Led
        ; warte 50 ms
        ldi    Timer2, LOW (5000)
        ldi    Timer3, HIGH(5000)
        rcall  Wait_10us_Timer2_Timer3
        rcall Flash_visible_Led
        ; warte 50 ms
        ldi    Timer2, LOW (5000)
        ldi    Timer3, HIGH(5000)
        rcall  Wait_10us_Timer2_Timer3
        rcall  Flash_visible_Led
        ret 

; ************ Flash IR Code **************************************
Flash_IR_Code:
        ; Schleife ber die gesammte Liste bis On Time = 0
        ldi    ZL, LOW (IR_CODE*2)
        ldi    ZH, HIGH(IR_CODE*2)

Flash_IR_Code_loop:
        ; Led 1 x blinken
        rcall  Flash_visible_Led

        ; hohle die Trgerfreqenz
        lpm    Timer2, Z+
        lpm    Timer3, Z+
        ; hohle ein Dummy
        lpm    Timer4, Z+
        lpm    Timer5, Z+
        
        ; Tabellenende erreicht (Trgerfreqenz MSB und LSB = 0)?
        clr    temp1		
        or     temp1, Timer2
        or     temp1, Timer3
        breq   Flash_IR_Code_exit       ; Ja, dann Ende

        ; in Timer3:Timer2 steht jetzt die Trgerfrequenz
        out    OCR1AL, Timer2
        out    OCR1AH, Timer3 

	    ; sende den Power Code fr dieses Gert	
        rcall  Flash_IR_Code_Segment
        
        ; 250 ms warte, damit die IR-Empfnger resetten
        ldi    Timer2, LOW(25000)
        ldi    Timer3, HIGH(25000)
        rcall  Wait_10us_Timer2_Timer3

        ; wiederhohle bis Tabellenende (fr alle Gerte)
        rjmp   Flash_IR_Code_loop

Flash_IR_Code_exit:		
        ; Fertig anzeigen 4 x Led
        rcall  Flash_visible_Led_4x
        ret

Flash_IR_Code_Segment:
        ; ** hohle beide Zeiten (An.- und Auszeit)
        ; hohle  Anzeit
        lpm    Timer2, Z+
        lpm    Timer3, Z+
        ; hohle Auszeit
        lpm    Timer4, Z+
        lpm    Timer5, Z+
        ; ** Segmentende erreicht (Auszeit beide Werte = 0)?
        clr    temp1		
        or     temp1, Timer4
        or     temp1, Timer5
        breq   Flash_IR_Code_Segment_exit ; Ja, dann Ende fr das Gert     

        ; ** Trgerfrequenz an
        ; start Timer1 outputting the carrier frequency to IR emitter on OC1A (PB1, pin 15)
        clr    temp1
        out    TCNT1H, temp1     ; Zhler auf Null 
        out    TCNT1L, temp1

		ldi    temp1, (1<<COM1A0)
                                 ; COM1A1:0=01 to toggle OC1A on Compare Match
                                 ; COM1B1:0=00 to disconnect OC1B
                                 ; WGM11:10=00 for CTC Mode(4) (WGM13:12=01 in TCCR0B)
        out    TCCR1A, temp1 
 
        ldi    temp1, (1<<WGM12) | (1<<CS10)  
		                         ; WGM13:12=01 for CTC Mode (WGM11:10=00 in TCCR0A)
                                 ; CS12:00=001 for divide by 1 prescaler
        out    TCCR1B, temp1 

        ; ** Anzeit abwarten
        rcall  Wait_10us_Timer2_Timer3 

        ; ** Trgerfrequenz aus
        clr    temp1
        out    TCCR1A, temp1 
        out    TCCR1B, temp1        ; Timer1 Stop 
        cbi    LED_PORT, IR_LED_PIN ; IR-Led aus

 		; ** Auszeit abwarten
        mov    Timer2, Timer4
        mov    Timer3, Timer5
        rcall  Wait_10us_Timer2_Timer3
        ; ** weiter fr das Gert
        rjmp   Flash_IR_Code_Segment 

Flash_IR_Code_Segment_exit:       
        ret

; ************ Code Tabellen **************************************
IR_CODE:
;.include "ami_code.asm"
.include "eu_code.asm"
; End of Table
.dw      0,         0
